/****************************************************************************
 * Copyright (c) 2010 Giorgio Sironi. All rights reserved.
 * This program and the accompanying materials are made available under 
 * the terms of the Eclipse Public License v1.0 which accompanies this 
 * distribution, and is available at http://www.eclipse.org/legal/epl-v10.html
 ****************************************************************************/

/****************************************************************************
 * Modified by Lorenzo Eccher to move from 0.8 to 0.9 of Eclipse.
 * This program and the accompanying materials are made available under 
 * the terms of the Eclipse Public License v1.0 which accompanies this 
 * distribution, and is available at http://www.eclipse.org/legal/epl-v10.html
 ****************************************************************************/


package smila.hackatlon.chansonnier.agent.youtube;


import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.connectivity.ConnectivityId;
import org.eclipse.smila.connectivity.framework.AbstractAgent;
import org.eclipse.smila.connectivity.framework.AgentException;
import org.eclipse.smila.connectivity.framework.util.AgentThreadState;
import org.eclipse.smila.connectivity.framework.util.ConnectivityIdFactory;
import org.eclipse.smila.datamodel.DataFactory;
import org.eclipse.smila.datamodel.Record;

import smila.hackatlon.chansonnier.utils.URLUtils;
import smila.hackatlon.chansonnier.agent.youtube.messages.Process;


class YouTubeLinkGrabberAgent extends AbstractAgent {
	private Queue<String> _linksToProcess = new LinkedList<String>();
    private final Log _log = LogFactory.getLog(YouTubeLinkGrabberAgent.class);
	private Map<String, String> _htmlPages = new HashMap<String, String>();
	/**
	 * Constant to compute millisecond values.
	 */
	private static final long MILLI_SECOND_MULTIPLIER = 1000;
	/**
	 * The _attributes.
	 */
	/**
	   * The sleepTime.
	   */
	private long _sleepTime;

	@Override
	protected void initialize() throws AgentException {
		// convert sleep time to milliseconds
	    _sleepTime = ((Process) getConfig().getProcess()).getSleepTime().longValue() * MILLI_SECOND_MULTIPLIER;
	    // read in configured attributes/attachments
	}

	@Override
	public void run() {
		try {
			while (!isStopThread()) {
				if (!_linksToProcess.isEmpty()) {
					final StringBuffer hash = new StringBuffer();
					
					String newLink = _linksToProcess.poll();
					Record record = createRecord(newLink);
					if (_log.isTraceEnabled()) {
						_log.trace("created record with Id " + record.getId());
					}
					String html = getHtmlPage(newLink);
					YouTubePageRecordImpl ytrecord = new YouTubePageRecordImpl(record,html);
					//Prepare readed some 
					/**
					_log.debug("video downloading");
					ytrecord.setVideo(getOriginal(_grabber.getVideo(newLink)));
					_log.debug("video downloaded");
					*/
					final long recordAttachmentLength = getAttachmentsByteLength(record);
					getCounterHelper().incrementAttachmentBytes(recordAttachmentLength);
					getCounterHelper().incrementRecords();

					// send record to AgentController
					getControllerCallback().add(getSessionId(), getConfig().getDeltaIndexing(), record, hash.toString());

					// wait configured sleepTime
					Thread.sleep(_sleepTime);
				}
			} // while
		} catch (Throwable t) {
			getAgentState().setLastError(t);
			getAgentState().setState(AgentThreadState.Aborted);
			throw new RuntimeException(t);
		} finally {
			try {
				stop();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

	}

	private Record createRecord(final String link) throws AgentException {
		// compute id
		final ConnectivityId id = ConnectivityIdFactory.getInstance().createId(getConfig().getDataSourceID(), link);
		// create record
		final Record record = DataFactory.DEFAULT.createRecord();
		record.setId(id.getObjectId());
		record.setSource(id.getDataSourceId());
		return record;
	}

	private String getHtmlPage(String link) throws IOException {
		if (!_htmlPages.containsKey(link)) {
			String html = URLUtils.retrieve(new URL(link));
			_htmlPages.put(link, html);
		}
		return _htmlPages.get(link);
	}
}
