package smila.hackatlon.chansonnier.agent.youtube;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.smila.datamodel.Record;

import smila.hackatlon.chansonnier.datamodel.CubrikRecord;

public class YouTubePageRecordImpl extends CubrikRecord{
	public enum Attachment implements IAttachment{
		htmlContent;
	}
	public enum Field implements IField{
		url,link,title,description,keywords;
	}
		
	protected String html;
	/**
	 * This costructor should be used by agents/crowlers
	 * @param r The new Record object to use/fill
	 * @param htmlcontent The html youtube page as readed by youtube api
	 */
	public YouTubePageRecordImpl(Record r,String htmlcontent){
		super(r);
		html=htmlcontent;
		record.setAttachment(Attachment.htmlContent.name(), htmlcontent.getBytes());
		//Filling the fields not filled :)
		fillRecord();
	}
	
	protected void fillRecord(){
		readTitle();
		readDescription();
		readKeywords();
	}

	/**
	 * This method read from html code the Page Title and set it as same name field
	 * Because the Page Title could contain Autor and Title fileds, this method set up them too.
	 * @return
	 */
	private String readTitle(){
		String key = Field.title.name();
		if(!record.getMetadata().containsKey(key)){ 
			Pattern p = Pattern.compile("<title>([^<]+)</title>");
			Matcher m = p.matcher(html);
			m.find();
			String fullTitle = m.group(1);
			record.getMetadata().put(key,fullTitle.trim());
		}
		return record.getMetadata().getStringValue(key);
	}

	private String readDescription(){
		String key = Field.description.name();
		if(!record.getMetadata().containsKey(key)){
			Pattern p = Pattern.compile("<meta\\s+name\\s*=\\s*\"description\"\\s+content\\s*=\\s*\"([^\"]+)\"");
			Matcher m = p.matcher(html);
			m.find();
			String description = m.group(1);
			record.getMetadata().put(key, description.trim());
		}
		return record.getMetadata().getStringValue(key);
	}

	private String readKeywords(){
		String key = Field.keywords.name();
		if(!record.getMetadata().containsKey(key)){
			Pattern p = Pattern.compile("<meta\\s+name=\\s*\"keywords\"\\s+content=\\s*\"([^\"]+)\"");
			Matcher m = p.matcher(html);
			m.find();
			String keywords = m.group(1);
			record.getMetadata().put(key,keywords.trim());
		}
		return record.getMetadata().getStringValue(key);
	}
	
	public String getUrl() {
		return getString(Field.url);
	}

	public void setUrl(String url) {
		put(Field.url,url);
	}

	public String getLink() {
		return getString(Field.link);
	}

	public void setLink(String link) {
		put(Field.link,link);
	}

	public String getTitle() {
		return getString(Field.title);
	}

	public void setTitle(String pageTitle) {
		put(Field.title,pageTitle);
	}

	public String getDescription() {
		return getString(Field.description);
	}

	public void setDescription(String description) {
		put(Field.description,description);
	}

	public String getKeywords() {
		return getString(Field.keywords);
	}

	public void setKeywords(String keywords) {
		put(Field.keywords,keywords);
	}
	
	public String getHtmlContent(){
		return new String(get(Attachment.htmlContent));
	}
	
	public void setHtmlContent(String htmlContent){
		put(Attachment.htmlContent,htmlContent.getBytes());
	}
	
	@Override
	public String getSourceName() {
		return "YouTubePage";
	}

	
}
