/****************************************************************************
 * Copyright (c) 2010 Giorgio Sironi. All rights reserved.
 * This program and the accompanying materials are made available under 
 * the terms of the Eclipse Public License v1.0 which accompanies this 
 * distribution, and is available at http://www.eclipse.org/legal/epl-v10.html
 ****************************************************************************/
package smila.hackatlon.chansonnier.utils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.URL;

public class URLUtils {
	static final int LENGTH = 256;
	
	/**
	 *  Generally HTTP or HTTPS
	 */
	private static String proxyType=null;
	/** 
	 * The name of the proxy Es: proxy.eng.it
	 */
	private static String proxyName=null;
	/**
	 *  The port used Es: 3128
	 */
	private static String proxyPort=null;
	/**
	 *  The base 64 encoded string for proxy auth user:password
	 * sun.misc.BASE64Encoder encoder = new sun.misc.BASE64Encoder();
	 * String proxyAuth = encoder.encode(new String("user:password").getBytes());
	 */
	private static String proxyB64Auth = null;
	
	static{
		proxyType=System.getProperty("proxy.type");
		proxyName=System.getProperty("proxy.name");
		proxyPort=System.getProperty("proxy.port");
		proxyB64Auth = System.getProperty("proxy.b64auth");
	}

	public static String retrieve(URL url) throws IOException {
		HttpURLConnection conn;
		if(proxyName!=null){
			Proxy proxy = new Proxy(Proxy.Type.valueOf(proxyType), new InetSocketAddress(proxyName, Integer.parseInt(proxyPort)));
			conn = (HttpURLConnection) url.openConnection(proxy);
			if(proxyB64Auth!=null){
				conn.setRequestProperty("Proxy-Authorization", "Basic " + proxyB64Auth);
			}
		}else{
			conn = (HttpURLConnection) url.openConnection();
		}
		conn.setRequestMethod("GET");
	    conn.connect();		
		InputStream is = conn.getInputStream();
	    ByteArrayOutputStream output = new ByteArrayOutputStream();
	    byte[] buffer = new byte[1024];
	    for (int bytesRead = 0; (bytesRead = is.read(buffer)) != -1; ) {
	      output.write(buffer, 0, bytesRead);
	    }
	    return output.toString();
	}
	
	public static String readStart(InputStream is) throws Exception {
		byte[] bytes = new byte[LENGTH];
		int len = is.read(bytes, 0, LENGTH);
		if (len != LENGTH) {
			throw new IOException("Length read was not sufficient.");
		}
		return new String(bytes);    
	}

	public static String readStart(File fp) throws Exception {
		return readStart(new FileInputStream(fp));
	}

	public static String readStart(String filename) throws Exception {
		File fp = new File(filename);
		return readStart(fp);
	}

	public static String httpEscape(String value) {
		return value.replace("\n+", " ").trim().replaceAll(" ", "%20");
	}
}
